# sb::App::Counter
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Counter;
use strict;
use sb::Plugin ();
use sb::Lock ();
use sb::Time ();
use sb::Text ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.05';
# 0.05 [2007/08/14] modified for Serene Bach 3.*
# 0.04 [2006/11/12] changed _update_count to fix a bug
# 0.03 [2005/08/11] changed _update_count to name log file with extension
# 0.02 [2005/07/10] changed _update_count to check existance of a file correctly
# 0.01 [2005/07/06] added entry point for plugin
# 0.00 [2004/02/01] generated
# --------------------------------------
# // defines / static variables
sub SECS_PER_MIN (){ 60 };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'result' => undef, # output content
		'type'   => undef, # output content type
		'update' => 0,     # update flag
		@_
	);
	$self->generate_accessor('result','type','update');
	$self->update( ($self->cgi->value('disp') ne 'on') ); # initialize update flag
	return $self;
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	my @funcs = sb::Plugin->load_extra_module('counter');
	push(@funcs,\&_default_handler); # default handler always runs at last
	foreach my $func (@funcs)
	{
		eval{ &$func($self); };
	}
	$self->cgi->head(
		'type'   => $self->type(),
		'length' => length($self->result()),
		'cache'  => $self->time->data() - 100,
	);
	binmode(STDOUT); # make sure STDOUT is binary mode
	$self->output($self->result());
}
# --------------------------------------
# // private functions - handlers
sub _default_handler
{
	my $self = shift;
	$self->_update_count() if ($self->update());
	return if ($self->type() or $self->result());
	if ($self->cgi->value('disp') eq 'on')
	{
		$self->type('text/javascript');
		$self->result(sprintf('document.write("%d");',$self->_load_count() ));
		return;
	}
	$self->type('image/gif');
	$self->result($self->_blank_image());
}
sub _update_count
{
	my $self = shift;
	my $cook = $self->cgi->cookie('name'=>$self->conf->value('basic_logtag'));
	my $date = $self->time->format('form'=>'%Year%%Mon%%Day%');
	my $lock = sb::Lock->lock('basename'=>$self->conf->value('file_lckcnt'));
	my $dir  = $self->conf->value('dir_data');
	my $update = undef;
	my ($count,$ip,$access);
	my %env = %{$self->cgi->remote_host()};
	$env{'refe'} = $self->cgi->referer();
	$env{'agnt'} = $self->cgi->user_agent();
	$env{'href'} = $self->cgi->value('href');
	$env{'quer'} = $self->cgi->value('refe');
	foreach my $key ( keys(%env) )
	{
		$env{$key} =~ s/\\/\\\\/g;
		$env{$key} =~ s/\t/\\t/g;
		$env{$key} =~ s/\n/\\n/g;
	}
	# load and save counter
	my $lfh = sb::Lock->locked_open($dir . $self->conf->value('file_access'),'without_truncate');
	if ($lfh)
	{
		my $line  = <$lfh>;
		($count,$ip,$access) = split("\t",$line); # count/ip/time
		$count = int($count); # to make sure $count is integer variable.
		$count++ if ($env{'addr'} ne $ip and $cook->{'check'} ne $self->conf->value('basic_cookiekey'));
		if ($self->conf->value('basic_min_update') == 0)
		{
			$access = $self->time->data();
		}
		elsif ($self->time->data() - $access > $self->conf->value('basic_min_update') * SECS_PER_MIN)
		{
			$access = $self->time->data();
			$update = 1;
		}
		truncate($lfh, 0);
		seek($lfh, 0, 0);
		print {$lfh} $count,"\t",$env{'addr'},"\t",$access,"\n";
		close($lfh);
	}
	# store log data
	my $file = $dir . $self->conf->value('dir_access') . $date . $self->conf->value('file_suf');
	my $flag = (-e $file) ? open(ACCESSLOG,">>$file") : open(ACCESSLOG,">$file");
	if ($flag and $cook->{'check'} ne $self->conf->value('basic_cookiekey'))
	{
		print ACCESSLOG $self->time->data(),"\t";
		print ACCESSLOG $env{'addr'},"\t";
		print ACCESSLOG $env{'host'},"\t";
		print ACCESSLOG $env{'refe'},"\t";
		print ACCESSLOG $env{'agnt'},"\t";
		print ACCESSLOG $env{'href'},"\t";
		print ACCESSLOG $env{'quer'},"\t","\n";
		close(ACCESSLOG);
		chmod($self->conf->value('basic_file_attr'),$file);
	}
	# trigger updating weblog
	$self->_update_weblog if ($update);
	$lock->unlock if ($lock);
}
# --------------------------------------
# // private functions - utilities
sub _load_count
{
	my $self = shift;
	my $file = $self->conf->value('dir_data') . $self->conf->value('file_access');
	open(TOTALIN,"<$file");
	my $count = <TOTALIN>;
	close(TOTALIN);
	return int($count);
}
sub _blank_image
{
	my $self = shift;
	return join('', map { pack('C*',hex($_)) }(
		'47','49','46','38','39','61','02','00','02','00','80','00','00','00','00',
		'00','ff','ff','ff','21','f9','04','01','00','00','01','00','2c','00','00',
		'00','00','02','00','02','00','00','02','02','8c','53','00','3b'
	));
}
1;
__END__
