/*!
  spica javascript libraries - entryeditor.js
  == written by Takuya Otani <takuya.otani@gmail.com> ===
  == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==
*/

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.00 [2008/11/01] the first attempt

Spica.Library.require('element','pickuplist','adminutils');
Spica.EntryEditor = function()
{
	this.form = document.mainform;
	// current time
	this.setTimeButton = Spica.$('set_nowtime');
	// upload form
	this.entryBodies = ['body','more'];
	// related categories
	this.entryCategory = Spica.$('entry_category');
	this.togglePickup = null;
	this.pickupList = null;
	this.initialize();
	return this;
};
Spica.EntryEditor.instance = null;
Spica.EntryEditor.prototype = {
	initialize : function()
	{
		var self = this;
		if (self.setTimeButton)
		{ // configure set current time button
			Spica.Event.register(self.setTimeButton,'click',self.setCurrentTime());
		}
 		for (var i=0,n=self.entryBodies.length;i<n;i++)
		{ // configure show upload form button
			var button = Spica.$('insert_' + self.entryBodies[i]);
			if (!button) continue;
			Spica.Event.register(button,'click',self.showUploadForm(button,self.entryBodies[i]));
		}
		if (self.entryCategory)
		{ // setup pickuplist for related categories
			self.pickupList = new Spica.PickupList('category');
			self.pickupList.setupMover();
			self.pickupList.bindForm(self.form);
			self.onChangeCategorySelector();
			Spica.Event.register(self.entryCategory,'change',function() { self.onChangeCategorySelector(); });
			// button to toggle the pickup list
			var button = document.createElement('input');
			button.type = 'button';
			button.value = Spica.$_('Related categories ...');
			button.id = 'show_category';
			Spica.Event.register(button,'click',function() { self.togglePickupList(button); });
			Spica.DOM.insertAfter(button, self.entryCategory);
		} // end of if (self.entryCategory)
	},
	setCurrentTime : function()
	{
		var form = this.form;
		return function() {
			var now = new Date();
			var zone = (form.entry_tz_hour.value * 60);
			zone = (zone < 0) 
				? zone - parseInt(form.entry_tz_min.value) 
				: zone + parseInt(form.entry_tz_min.value);
			now.setTime(now.getTime() + zone * 60 * 1000);
			var dateSet = {
				year : now.getUTCFullYear(),
				mon  : now.getUTCMonth() + 1,
				day  : now.getUTCDate(),
				hour : now.getUTCHours(),
				min  : now.getUTCMinutes(),
				sec  : now.getUTCSeconds()
			};
			if (dateSet.year < 2000) dateSet.year += 1900;
			for (var key in dateSet)
			{
				form['entry_date_' + key].value = (dateSet[key] < 10) ? '0' + dateSet[key] : dateSet[key];
			}
			Spica.FormHandler.instance.setConfirmationToLeave(true);
		};
	},
	showUploadForm : function(button, target)
	{
		var self = this;
		var form = Spica.$('edit_addfile');
		if (!form) return function() {};
		return function() {
			var field = Spica.DOM.getParent(Spica.DOM.getParent(button));
			if (field) field.appendChild(form);
			var current = Spica.$('insert_target');
			if (!current)
			{
				form.style.display = 'block';
			}
			else if (current.value == target && form.style.display == 'block')
			{
				form.style.display = 'none';
			}
			else
			{
				form.style.display = 'block';
				current.value = target;
			}
		};
	},
	onChangeCategorySelector : function()
	{
		var self = this;
		var current_category = 'none';
		for (var i=0,n=self.entryCategory.options.length;i<n;i++)
		{
			var opt = self.entryCategory.options[i];
			if (opt.value == 'none') continue;
			if (opt.selected) current_category = opt.value;
			self.pickupList.setCandidate({value: opt.value, text: opt.text});
		}
		if (current_category != 'none')
			self.pickupList.setIgnored([current_category]);
		self.pickupList.updateList();
	},
	togglePickupList : function(button)
	{
		var self = this;
		var list = document.getElementById('multi_category');
		if (!list) return;
		list.style.display = (list.style.display == 'none') 
			? 'block' 
			: 'none';
		button.value = (list.style.display == 'none')
			? Spica.$_('Related categories ...') 
			: Spica.$_('Hide option');
	},
	sbit : function(txt_subj,txt_url,txt_body)
	{
		if (txt_subj.length > 0)
		{
			var obj = Spica.$('entry_title');
			txt_subj = txt_subj.replace(/\n/g,"");
			obj.value = 'Re: ' + unescape(txt_subj);
		}
		if (txt_body.length > 0)
		{
			var obj = Spica.$('entry_body');
			obj.value += '<blockquote><p>' + unescape(txt_body) + '</p></blockquote>';
		}
		if (txt_url.length > 0 && txt_subj.length > 0)
		{
			var obj = Spica.$('entry_body');
			obj.value += '<p class="source"><cite><a href="' + unescape(txt_url) + '">' + unescape(txt_subj) + '</a></cite></p>\n';
		}
	}
};
Spica.EntryEditor.getInstance = function()
{
	if (!Spica.EntryEditor.instance)
		Spica.EntryEditor.instance = new Spica.EntryEditor();
	return Spica.EntryEditor.instance;
}
Spica.Event.run(function() { Spica.EntryEditor.getInstance(); });
